@@ -16,6 +16,7 @@
 # limitations under the License.
 ################################################################################
 import warnings
+from typing import Dict, List, Optional
 
 from py4j.java_gateway import java_import
 
@@ -36,14 +37,7 @@ class Catalog(object):
     def __init__(self, j_catalog):
         self._j_catalog = j_catalog
 
-    @staticmethod
-    def _get(j_catalog):
-        if j_catalog.getClass().getName() == "org.apache.flink.table.catalog.hive.HiveCatalog":
-            return HiveCatalog(j_hive_catalog=j_catalog)
-        else:
-            return Catalog(j_catalog)
-
-    def get_default_database(self):
+    def get_default_database(self) -> str:
         """
         Get the name of the default database for this catalog. The default database will be the
         current database for the catalog when user's session doesn't specify a current database.
@@ -55,7 +49,7 @@ class Catalog(object):
         """
         return self._j_catalog.getDefaultDatabase()
 
-    def list_databases(self):
+    def list_databases(self) -> List[str]:
         """
         Get the names of all databases in this catalog.
 
@@ -64,7 +58,7 @@ class Catalog(object):
         """
         return list(self._j_catalog.listDatabases())
 
-    def get_database(self, database_name):
+    def get_database(self, database_name: str) -> 'CatalogDatabase':
         """
         Get a database from this catalog.
 
@@ -75,7 +69,7 @@ class Catalog(object):
         """
         return CatalogDatabase._get(self._j_catalog.getDatabase(database_name))
 
-    def database_exists(self, database_name):
+    def database_exists(self, database_name: str) -> bool:
         """
         Check if a database exists in this catalog.
 
@@ -85,7 +79,7 @@ class Catalog(object):
         """
         return self._j_catalog.databaseExists(database_name)
 
-    def create_database(self, name, database, ignore_if_exists):
+    def create_database(self, name: str, database: 'CatalogDatabase', ignore_if_exists: bool):
         """
         Create a database.
 
@@ -101,7 +95,7 @@ class Catalog(object):
         """
         self._j_catalog.createDatabase(name, database._j_catalog_database, ignore_if_exists)
 
-    def drop_database(self, name, ignore_if_exists):
+    def drop_database(self, name: str, ignore_if_exists: bool):
         """
         Drop a database.
 
@@ -114,7 +108,8 @@ class Catalog(object):
         """
         self._j_catalog.dropDatabase(name, ignore_if_exists)
 
-    def alter_database(self, name, new_database, ignore_if_not_exists):
+    def alter_database(self, name: str, new_database: 'CatalogDatabase',
+                       ignore_if_not_exists: bool):
         """
         Modify an existing database.
 
@@ -129,7 +124,7 @@ class Catalog(object):
         """
         self._j_catalog.alterDatabase(name, new_database._j_catalog_database, ignore_if_not_exists)
 
-    def list_tables(self, database_name):
+    def list_tables(self, database_name: str) -> List[str]:
         """
         Get names of all tables and views under this database. An empty list is returned if none
         exists.
@@ -141,7 +136,7 @@ class Catalog(object):
         """
         return list(self._j_catalog.listTables(database_name))
 
-    def list_views(self, database_name):
+    def list_views(self, database_name: str) -> List[str]:
         """
         Get names of all views under this database. An empty list is returned if none exists.
 
@@ -152,7 +147,7 @@ class Catalog(object):
         """
         return list(self._j_catalog.listViews(database_name))
 
-    def get_table(self, table_path):
+    def get_table(self, table_path: 'ObjectPath') -> 'CatalogBaseTable':
         """
         Get a CatalogTable or CatalogView identified by tablePath.
 
@@ -163,7 +158,7 @@ class Catalog(object):
         """
         return CatalogBaseTable._get(self._j_catalog.getTable(table_path._j_object_path))
 
-    def table_exists(self, table_path):
+    def table_exists(self, table_path: 'ObjectPath') -> bool:
         """
         Check if a table or view exists in this catalog.
 
@@ -173,7 +168,7 @@ class Catalog(object):
         """
         return self._j_catalog.tableExists(table_path._j_object_path)
 
-    def drop_table(self, table_path, ignore_if_not_exists):
+    def drop_table(self, table_path: 'ObjectPath', ignore_if_not_exists: bool):
         """
         Drop a table or view.
 
@@ -186,7 +181,8 @@ class Catalog(object):
         """
         self._j_catalog.dropTable(table_path._j_object_path, ignore_if_not_exists)
 
-    def rename_table(self, table_path, new_table_name, ignore_if_not_exists):
+    def rename_table(self, table_path: 'ObjectPath', new_table_name: str,
+                     ignore_if_not_exists: bool):
         """
         Rename an existing table or view.
 
@@ -200,7 +196,8 @@ class Catalog(object):
         """
         self._j_catalog.renameTable(table_path._j_object_path, new_table_name, ignore_if_not_exists)
 
-    def create_table(self, table_path, table, ignore_if_exists):
+    def create_table(self, table_path: 'ObjectPath', table: 'CatalogBaseTable',
+                     ignore_if_exists: bool):
         """
         Create a new table or view.
 
@@ -217,7 +214,8 @@ class Catalog(object):
         self._j_catalog.createTable(table_path._j_object_path, table._j_catalog_base_table,
                                     ignore_if_exists)
 
-    def alter_table(self, table_path, new_table, ignore_if_not_exists):
+    def alter_table(self, table_path: 'ObjectPath', new_table: 'CatalogBaseTable',
+                    ignore_if_not_exists):
         """
         Modify an existing table or view.
         Note that the new and old CatalogBaseTable must be of the same type. For example,
@@ -235,7 +233,10 @@ class Catalog(object):
         self._j_catalog.alterTable(table_path._j_object_path, new_table._j_catalog_base_table,
                                    ignore_if_not_exists)
 
-    def list_partitions(self, table_path, partition_spec=None):
+    def list_partitions(self,
+                        table_path: 'ObjectPath',
+                        partition_spec: 'CatalogPartitionSpec' = None)\
+            -> List['CatalogPartitionSpec']:
         """
         Get CatalogPartitionSpec of all partitions of the table.
 
@@ -253,7 +254,8 @@ class Catalog(object):
             return [CatalogPartitionSpec(p) for p in self._j_catalog.listPartitions(
                 table_path._j_object_path, partition_spec._j_catalog_partition_spec)]
 
-    def get_partition(self, table_path, partition_spec):
+    def get_partition(self, table_path: 'ObjectPath', partition_spec: 'CatalogPartitionSpec') \
+            -> 'CatalogPartition':
         """
         Get a partition of the given table.
         The given partition spec keys and values need to be matched exactly for a result.
@@ -267,7 +269,8 @@ class Catalog(object):
         return CatalogPartition._get(self._j_catalog.getPartition(
             table_path._j_object_path, partition_spec._j_catalog_partition_spec))
 
-    def partition_exists(self, table_path, partition_spec):
+    def partition_exists(self, table_path: 'ObjectPath',
+                         partition_spec: 'CatalogPartitionSpec') -> bool:
         """
         Check whether a partition exists or not.
 
@@ -280,7 +283,8 @@ class Catalog(object):
         return self._j_catalog.partitionExists(
             table_path._j_object_path, partition_spec._j_catalog_partition_spec)
 
-    def create_partition(self, table_path, partition_spec, partition, ignore_if_exists):
+    def create_partition(self, table_path: 'ObjectPath', partition_spec: 'CatalogPartitionSpec',
+                         partition: 'CatalogPartition', ignore_if_exists: bool):
         """
         Create a partition.
 
@@ -302,7 +306,8 @@ class Catalog(object):
                                         partition._j_catalog_partition,
                                         ignore_if_exists)
 
-    def drop_partition(self, table_path, partition_spec, ignore_if_not_exists):
+    def drop_partition(self, table_path: 'ObjectPath', partition_spec: 'CatalogPartitionSpec',
+                       ignore_if_not_exists: bool):
         """
         Drop a partition.
 
@@ -319,7 +324,8 @@ class Catalog(object):
                                       partition_spec._j_catalog_partition_spec,
                                       ignore_if_not_exists)
 
-    def alter_partition(self, table_path, partition_spec, new_partition, ignore_if_not_exists):
+    def alter_partition(self, table_path: 'ObjectPath', partition_spec: 'CatalogPartitionSpec',
+                        new_partition: 'CatalogPartition', ignore_if_not_exists: bool):
         """
         Alter a partition.
 
@@ -338,7 +344,7 @@ class Catalog(object):
                                        new_partition._j_catalog_partition,
                                        ignore_if_not_exists)
 
-    def list_functions(self, database_name):
+    def list_functions(self, database_name: str) -> List[str]:
         """
         List the names of all functions in the given database. An empty list is returned if none is
         registered.
@@ -350,7 +356,7 @@ class Catalog(object):
         """
         return list(self._j_catalog.listFunctions(database_name))
 
-    def get_function(self, function_path):
+    def get_function(self, function_path: 'ObjectPath') -> 'CatalogFunction':
         """
         Get the function.
 
@@ -361,7 +367,7 @@ class Catalog(object):
         """
         return CatalogFunction._get(self._j_catalog.getFunction(function_path._j_object_path))
 
-    def function_exists(self, function_path):
+    def function_exists(self, function_path: 'ObjectPath') -> bool:
         """
         Check whether a function exists or not.
 
@@ -371,7 +377,8 @@ class Catalog(object):
         """
         return self._j_catalog.functionExists(function_path._j_object_path)
 
-    def create_function(self, function_path, function, ignore_if_exists):
+    def create_function(self, function_path: 'ObjectPath', function: 'CatalogFunction',
+                        ignore_if_exists: bool):
         """
         Create a function.
 
@@ -389,7 +396,8 @@ class Catalog(object):
                                        function._j_catalog_function,
                                        ignore_if_exists)
 
-    def alter_function(self, function_path, new_function, ignore_if_not_exists):
+    def alter_function(self, function_path: 'ObjectPath', new_function: 'CatalogFunction',
+                       ignore_if_not_exists: bool):
         """
         Modify an existing function.
 
@@ -405,7 +413,7 @@ class Catalog(object):
                                       new_function._j_catalog_function,
                                       ignore_if_not_exists)
 
-    def drop_function(self, function_path, ignore_if_not_exists):
+    def drop_function(self, function_path: 'ObjectPath', ignore_if_not_exists: bool):
         """
         Drop a function.
 
@@ -418,7 +426,7 @@ class Catalog(object):
         """
         self._j_catalog.dropFunction(function_path._j_object_path, ignore_if_not_exists)
 
-    def get_table_statistics(self, table_path):
+    def get_table_statistics(self, table_path: 'ObjectPath') -> 'CatalogTableStatistics':
         """
         Get the statistics of a table.
 
@@ -431,7 +439,7 @@ class Catalog(object):
             j_catalog_table_statistics=self._j_catalog.getTableStatistics(
                 table_path._j_object_path))
 
-    def get_table_column_statistics(self, table_path):
+    def get_table_column_statistics(self, table_path: 'ObjectPath') -> 'CatalogColumnStatistics':
         """
         Get the column statistics of a table.
 
@@ -444,7 +452,10 @@ class Catalog(object):
             j_catalog_column_statistics=self._j_catalog.getTableColumnStatistics(
                 table_path._j_object_path))
 
-    def get_partition_statistics(self, table_path, partition_spec):
+    def get_partition_statistics(self,
+                                 table_path: 'ObjectPath',
+                                 partition_spec: 'CatalogPartitionSpec') \
+            -> 'CatalogTableStatistics':
         """
         Get the statistics of a partition.
 
@@ -458,7 +469,10 @@ class Catalog(object):
             j_catalog_table_statistics=self._j_catalog.getPartitionStatistics(
                 table_path._j_object_path, partition_spec._j_catalog_partition_spec))
 
-    def get_partition_column_statistics(self, table_path, partition_spec):
+    def get_partition_column_statistics(self,
+                                        table_path: 'ObjectPath',
+                                        partition_spec: 'CatalogPartitionSpec') \
+            -> 'CatalogColumnStatistics':
         """
         Get the column statistics of a partition.
 
@@ -472,7 +486,10 @@ class Catalog(object):
             j_catalog_column_statistics=self._j_catalog.getPartitionColumnStatistics(
                 table_path._j_object_path, partition_spec._j_catalog_partition_spec))
 
-    def alter_table_statistics(self, table_path, table_statistics, ignore_if_not_exists):
+    def alter_table_statistics(self,
+                               table_path: 'ObjectPath',
+                               table_statistics: 'CatalogTableStatistics',
+                               ignore_if_not_exists: bool):
         """
         Update the statistics of a table.
 
@@ -489,7 +506,10 @@ class Catalog(object):
             table_statistics._j_catalog_table_statistics,
             ignore_if_not_exists)
 
-    def alter_table_column_statistics(self, table_path, column_statistics, ignore_if_not_exists):
+    def alter_table_column_statistics(self,
+                                      table_path: 'ObjectPath',
+                                      column_statistics: 'CatalogColumnStatistics',
+                                      ignore_if_not_exists: bool):
         """
         Update the column statistics of a table.
 
@@ -506,8 +526,11 @@ class Catalog(object):
             column_statistics._j_catalog_column_statistics,
             ignore_if_not_exists)
 
-    def alter_partition_statistics(self, table_path, partition_spec, partition_statistics,
-                                   ignore_if_not_exists):
+    def alter_partition_statistics(self,
+                                   table_path: 'ObjectPath',
+                                   partition_spec: 'CatalogPartitionSpec',
+                                   partition_statistics: 'CatalogTableStatistics',
+                                   ignore_if_not_exists: bool):
         """
         Update the statistics of a table partition.
 
@@ -526,8 +549,11 @@ class Catalog(object):
             partition_statistics._j_catalog_table_statistics,
             ignore_if_not_exists)
 
-    def alter_partition_column_statistics(self, table_path, partition_spec, column_statistics,
-                                          ignore_if_not_exists):
+    def alter_partition_column_statistics(self,
+                                          table_path: 'ObjectPath',
+                                          partition_spec: 'CatalogPartitionSpec',
+                                          column_statistics: 'CatalogColumnStatistics',
+                                          ignore_if_not_exists: bool):
         """
         Update the column statistics of a table partition.
 
@@ -555,17 +581,34 @@ class CatalogDatabase(object):
     def __init__(self, j_catalog_database):
         self._j_catalog_database = j_catalog_database
 
+    @staticmethod
+    def create_instance(
+        properties: Dict[str, str],
+        comment: str = None
+    ) -> "CatalogDatabase":
+        """
+        Creates an instance of CatalogDatabase.
+
+        :param properties: Property of the database
+        :param comment: Comment of the database
+        """
+        assert properties is not None
+
+        gateway = get_gateway()
+        return CatalogDatabase(gateway.jvm.org.apache.flink.table.catalog.CatalogDatabaseImpl(
+            properties, comment))
+
     @staticmethod
     def _get(j_catalog_database):
         return CatalogDatabase(j_catalog_database)
 
-    def get_properties(self):
+    def get_properties(self) -> Dict[str, str]:
         """
         Get a map of properties associated with the database.
         """
         return dict(self._j_catalog_database.getProperties())
 
-    def get_comment(self):
+    def get_comment(self) -> str:
         """
         Get comment of the database.
 
@@ -573,7 +616,7 @@ class CatalogDatabase(object):
         """
         return self._j_catalog_database.getComment()
 
-    def copy(self):
+    def copy(self) -> 'CatalogDatabase':
         """
         Get a deep copy of the CatalogDatabase instance.
 
@@ -581,7 +624,7 @@ class CatalogDatabase(object):
         """
         return CatalogDatabase(self._j_catalog_database.copy())
 
-    def get_description(self):
+    def get_description(self) -> Optional[str]:
         """
         Get a brief description of the database.
 
@@ -593,7 +636,7 @@ class CatalogDatabase(object):
         else:
             return None
 
-    def get_detailed_description(self):
+    def get_detailed_description(self) -> Optional[str]:
         """
         Get a detailed description of the database.
 
@@ -615,6 +658,60 @@ class CatalogBaseTable(object):
     def __init__(self, j_catalog_base_table):
         self._j_catalog_base_table = j_catalog_base_table
 
+    @staticmethod
+    def create_table(
+        schema: TableSchema,
+        partition_keys: List[str] = [],
+        properties: Dict[str, str] = {},
+        comment: str = None
+    ) -> "CatalogBaseTable":
+        """
+        Create an instance of CatalogBaseTable for the catalog table.
+
+        :param schema: the table schema
+        :param partition_keys: the partition keys, default empty
+        :param properties: the properties of the catalog table
+        :param comment: the comment of the catalog table
+        """
+        assert schema is not None
+        assert partition_keys is not None
+        assert properties is not None
+
+        gateway = get_gateway()
+        return CatalogBaseTable(
+            gateway.jvm.org.apache.flink.table.catalog.CatalogTableImpl(
+                schema._j_table_schema, partition_keys, properties, comment))
+
+    @staticmethod
+    def create_view(
+        original_query: str,
+        expanded_query: str,
+        schema: TableSchema,
+        properties: Dict[str, str],
+        comment: str = None
+    ) -> "CatalogBaseTable":
+        """
+        Create an instance of CatalogBaseTable for the catalog view.
+
+        :param original_query: the original text of the view definition
+        :param expanded_query: the expanded text of the original view definition, this is needed
+                               because the context such as current DB is lost after the session,
+                               in which view is defined, is gone. Expanded query text takes care
+                               of the this, as an example.
+        :param schema: the table schema
+        :param properties: the properties of the catalog view
+        :param comment: the comment of the catalog view
+        """
+        assert original_query is not None
+        assert expanded_query is not None
+        assert schema is not None
+        assert properties is not None
+
+        gateway = get_gateway()
+        return CatalogBaseTable(
+            gateway.jvm.org.apache.flink.table.catalog.CatalogViewImpl(
+                original_query, expanded_query, schema._j_table_schema, properties, comment))
+
     @staticmethod
     def _get(j_catalog_base_table):
         return CatalogBaseTable(j_catalog_base_table)
@@ -632,7 +729,7 @@ class CatalogBaseTable(object):
         """
         return dict(self._j_catalog_base_table.getOptions())
 
-    def get_properties(self):
+    def get_properties(self) -> Dict[str, str]:
         """
         Get the properties of the table.
 
@@ -645,7 +742,7 @@ class CatalogBaseTable(object):
                       DeprecationWarning)
         return dict(self._j_catalog_base_table.getProperties())
 
-    def get_schema(self):
+    def get_schema(self) -> TableSchema:
         """
         Get the schema of the table.
 
@@ -653,7 +750,7 @@ class CatalogBaseTable(object):
         """
         return TableSchema(j_table_schema=self._j_catalog_base_table.getSchema())
 
-    def get_comment(self):
+    def get_comment(self) -> str:
         """
         Get comment of the table or view.
 
@@ -661,7 +758,7 @@ class CatalogBaseTable(object):
         """
         return self._j_catalog_base_table.getComment()
 
-    def copy(self):
+    def copy(self) -> 'CatalogBaseTable':
         """
         Get a deep copy of the CatalogBaseTable instance.
 
@@ -669,7 +766,7 @@ class CatalogBaseTable(object):
         """
         return CatalogBaseTable(self._j_catalog_base_table.copy())
 
-    def get_description(self):
+    def get_description(self) -> Optional[str]:
         """
         Get a brief description of the table or view.
 
@@ -681,7 +778,7 @@ class CatalogBaseTable(object):
         else:
             return None
 
-    def get_detailed_description(self):
+    def get_detailed_description(self) -> Optional[str]:
         """
         Get a detailed description of the table or view.
 
@@ -702,11 +799,29 @@ class CatalogPartition(object):
     def __init__(self, j_catalog_partition):
         self._j_catalog_partition = j_catalog_partition
 
+    @staticmethod
+    def create_instance(
+        properties: Dict[str, str],
+        comment: str = None
+    ) -> "CatalogPartition":
+        """
+        Creates an instance of CatalogPartition.
+
+        :param properties: Property of the partition
+        :param comment: Comment of the partition
+        """
+        assert properties is not None
+
+        gateway = get_gateway()
+        return CatalogPartition(
+            gateway.jvm.org.apache.flink.table.catalog.CatalogPartitionImpl(
+                properties, comment))
+
     @staticmethod
     def _get(j_catalog_partition):
         return CatalogPartition(j_catalog_partition)
 
-    def get_properties(self):
+    def get_properties(self) -> Dict[str, str]:
         """
         Get a map of properties associated with the partition.
 
@@ -714,7 +829,7 @@ class CatalogPartition(object):
         """
         return dict(self._j_catalog_partition.getProperties())
 
-    def copy(self):
+    def copy(self) -> 'CatalogPartition':
         """
         Get a deep copy of the CatalogPartition instance.
 
@@ -722,7 +837,7 @@ class CatalogPartition(object):
         """
         return CatalogPartition(self._j_catalog_partition.copy())
 
-    def get_description(self):
+    def get_description(self) -> Optional[str]:
         """
         Get a brief description of the partition object.
 
@@ -734,7 +849,7 @@ class CatalogPartition(object):
         else:
             return None
 
-    def get_detailed_description(self):
+    def get_detailed_description(self) -> Optional[str]:
         """
         Get a detailed description of the partition object.
 
@@ -746,12 +861,11 @@ class CatalogPartition(object):
         else:
             return None
 
-    def get_comment(self):
+    def get_comment(self) -> str:
         """
         Get comment of the partition.
 
         :return: Comment of the partition.
-        :rtype: str
         """
         return self._j_catalog_partition.getComment()
 
@@ -764,11 +878,39 @@ class CatalogFunction(object):
     def __init__(self, j_catalog_function):
         self._j_catalog_function = j_catalog_function
 
+    @staticmethod
+    def create_instance(
+        class_name: str,
+        function_language: str = 'Python'
+    ) -> "CatalogFunction":
+        """
+        Creates an instance of CatalogDatabase.
+
+        :param class_name: full qualified path of the class name
+        :param function_language: language of the function, must be one of
+                                  'Python', 'Java' or 'Scala'. (default Python)
+        """
+        assert class_name is not None
+
+        gateway = get_gateway()
+        FunctionLanguage = gateway.jvm.org.apache.flink.table.catalog.FunctionLanguage
+        if function_language.lower() == 'python':
+            function_language = FunctionLanguage.PYTHON
+        elif function_language.lower() == 'java':
+            function_language = FunctionLanguage.JAVA
+        elif function_language.lower() == 'scala':
+            function_language = FunctionLanguage.SCALA
+        else:
+            raise ValueError("function_language must be one of 'Python', 'Java' or 'Scala'")
+        return CatalogFunction(
+            gateway.jvm.org.apache.flink.table.catalog.CatalogFunctionImpl(
+                class_name, function_language))
+
     @staticmethod
     def _get(j_catalog_function):
         return CatalogFunction(j_catalog_function)
 
-    def get_class_name(self):
+    def get_class_name(self) -> str:
         """
         Get the full name of the class backing the function.
 
@@ -776,7 +918,7 @@ class CatalogFunction(object):
         """
         return self._j_catalog_function.getClassName()
 
-    def copy(self):
+    def copy(self) -> 'CatalogFunction':
         """
         Create a deep copy of the function.
 
@@ -784,7 +926,7 @@ class CatalogFunction(object):
         """
         return CatalogFunction(self._j_catalog_function.copy())
 
-    def get_description(self):
+    def get_description(self) -> Optional[str]:
         """
         Get a brief description of the function.
 
@@ -796,7 +938,7 @@ class CatalogFunction(object):
         else:
             return None
 
-    def get_detailed_description(self):
+    def get_detailed_description(self) -> Optional[str]:
         """
         Get a detailed description of the function.
 
@@ -808,7 +950,7 @@ class CatalogFunction(object):
         else:
             return None
 
-    def is_generic(self):
+    def is_generic(self) -> bool:
         """
         Whether or not is the function a flink UDF.
 
@@ -851,17 +993,17 @@ class ObjectPath(object):
         return isinstance(other, self.__class__) and self._j_object_path.equals(
             other._j_object_path)
 
-    def get_database_name(self):
+    def get_database_name(self) -> str:
         return self._j_object_path.getDatabaseName()
 
-    def get_object_name(self):
+    def get_object_name(self) -> str:
         return self._j_object_path.getObjectName()
 
-    def get_full_name(self):
+    def get_full_name(self) -> str:
         return self._j_object_path.getFullName()
 
     @staticmethod
-    def from_string(full_name):
+    def from_string(full_name: str) -> 'ObjectPath':
         gateway = get_gateway()
         return ObjectPath(j_object_path=gateway.jvm.ObjectPath.fromString(full_name))
 
@@ -890,7 +1032,7 @@ class CatalogPartitionSpec(object):
         return isinstance(other, self.__class__) and self._j_catalog_partition_spec.equals(
             other._j_catalog_partition_spec)
 
-    def get_partition_spec(self):
+    def get_partition_spec(self) -> Dict[str, str]:
         """
         Get the partition spec as key-value map.
 
@@ -918,34 +1060,34 @@ class CatalogTableStatistics(object):
         else:
             self._j_catalog_table_statistics = j_catalog_table_statistics
 
-    def get_row_count(self):
+    def get_row_count(self) -> int:
         """
         The number of rows in the table or partition.
         """
         return self._j_catalog_table_statistics.getRowCount()
 
-    def get_field_count(self):
+    def get_field_count(self) -> int:
         """
         The number of files on disk.
         """
         return self._j_catalog_table_statistics.getFileCount()
 
-    def get_total_size(self):
+    def get_total_size(self) -> int:
         """
         The total size in bytes.
         """
         return self._j_catalog_table_statistics.getTotalSize()
 
-    def get_raw_data_size(self):
+    def get_raw_data_size(self) -> int:
         """
         The raw data size (size when loaded in memory) in bytes.
         """
         return self._j_catalog_table_statistics.getRawDataSize()
 
-    def get_properties(self):
+    def get_properties(self) -> Dict[str, str]:
         return dict(self._j_catalog_table_statistics.getProperties())
 
-    def copy(self):
+    def copy(self) -> 'CatalogTableStatistics':
         """
         Create a deep copy of "this" instance.
         """
@@ -975,10 +1117,10 @@ class CatalogColumnStatistics(object):
     def get_column_statistics_data(self):
         return self._j_catalog_column_statistics.getColumnStatisticsData()
 
-    def get_properties(self):
+    def get_properties(self) -> Dict[str, str]:
         return dict(self._j_catalog_column_statistics.getProperties())
 
-    def copy(self):
+    def copy(self) -> 'CatalogColumnStatistics':
         return CatalogColumnStatistics(
             j_catalog_column_statistics=self._j_catalog_column_statistics.copy())
 
@@ -988,11 +1130,31 @@ class HiveCatalog(Catalog):
     A catalog implementation for Hive.
     """
 
-    def __init__(self, catalog_name=None, default_database="default", hive_conf_dir=None,
-                 j_hive_catalog=None):
+    def __init__(self, catalog_name: str, default_database: str = None, hive_conf_dir: str = None):
+        assert catalog_name is not None
+
         gateway = get_gateway()
 
-        if j_hive_catalog is None:
-            j_hive_catalog = gateway.jvm.org.apache.flink.table.catalog.hive.HiveCatalog(
-                catalog_name, default_database, hive_conf_dir)
+        j_hive_catalog = gateway.jvm.org.apache.flink.table.catalog.hive.HiveCatalog(
+            catalog_name, default_database, hive_conf_dir)
         super(HiveCatalog, self).__init__(j_hive_catalog)
+
+
+class JdbcCatalog(Catalog):
+    """
+    A catalog implementation for Jdbc.
+    """
+    def __init__(self, catalog_name: str, default_database: str, username: str, pwd: str,
+                 base_url: str):
+        assert catalog_name is not None
+        assert default_database is not None
+        assert username is not None
+        assert pwd is not None
+        assert base_url is not None
+
+        from pyflink.java_gateway import get_gateway
+        gateway = get_gateway()
+
+        j_jdbc_catalog = gateway.jvm.org.apache.flink.connector.jdbc.catalog.JdbcCatalog(
+            catalog_name, default_database, username, pwd, base_url)
+        super(JdbcCatalog, self).__init__(j_jdbc_catalog)
